﻿using System;
using System.Collections.Generic;
using System.Text;

namespace SQLSaturdayLibrary
{
    public static class LevenshteinDistance
    {

        private static int Minimum(int first, int second, int third)
        {
            int returnValue = 0;

            returnValue = first;
            if (second < returnValue)
            {
                returnValue = second;
            }
            if (third < returnValue)
            {
                returnValue = third;
            }

            return returnValue;
        }

        public static double EqualityPercent(string first, string second)
        {
            double returnValue = 0;
            if (!(string.IsNullOrEmpty(first) && string.IsNullOrEmpty(second)))
            {
                int distance = Distance(first, second);
                int firstLength = first.Length;
                int secondLength = second.Length;
                if (secondLength > firstLength)
                {
                    returnValue = (double)distance / (double)secondLength;
                }
                else if (secondLength < firstLength)
                {
                    returnValue = (double)distance / (double)firstLength;
                }
                else if (secondLength == firstLength && secondLength >= 0)
                {
                    returnValue = (double)distance / (double)firstLength;
                }
                return (1D - returnValue);
            }
            else if (string.IsNullOrEmpty(first) && string.IsNullOrEmpty(second))
                return 1;
            else
                return 0;
        }

        public static int Distance(string first, string second)
        {
            int tempDistance = 0;
            int[,] letterMatrix = null; // matrix
            int secondLength = 0; // length of second string
            int firstLength = 0; // length of first string
            int firstCounter = 0; // iterates through first string
            int secondCounter = 0; // iterates through second string
            string first_i = null; // ith character of first string
            string second_j = null; // jth character of second string
            int cost = 0; // cost

            // Step 1

            firstLength = first.Length;
            secondLength = second.Length;
            letterMatrix = new int[firstLength + 1, secondLength + 1];
            if (firstLength == 0)
            {
                return secondLength;
            }
            if (secondLength == 0)
            {
                return firstLength;
            }

            // Step 2

            for (firstCounter = 0; firstCounter <= firstLength; firstCounter++)
            {
                letterMatrix[firstCounter, 0] = firstCounter;
            }

            for (secondCounter = 0; secondCounter <= secondLength; secondCounter++)
            {
                letterMatrix[0, secondCounter] = secondCounter;
            }

            // Step 3

            for (firstCounter = 1; firstCounter <= firstLength; firstCounter++)
            {

                first_i = first.Substring(firstCounter - 1, 1);

                // Step 4

                for (secondCounter = 1; secondCounter <= secondLength; secondCounter++)
                {

                    second_j = second.Substring(secondCounter - 1, 1);

                    // Step 5

                    if (first_i == second_j)
                    {
                        cost = 0;
                    }
                    else
                    {
                        cost = 1;
                    }

                    // Step 6

                    letterMatrix[firstCounter, secondCounter] = Minimum(letterMatrix[firstCounter - 1, secondCounter] + 1, letterMatrix[firstCounter, secondCounter - 1] + 1, letterMatrix[firstCounter - 1, secondCounter - 1] + cost);

                }

            }

            // Step 7

            tempDistance = letterMatrix[firstLength, secondLength];
            letterMatrix = null;

            return tempDistance;
        }
    }


}
